# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_events', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='slug',
            field=models.SlugField(max_length=100, unique_for_date='start'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.conf.urls import url

from . import views


calendar_urlpatterns = [
    # Calendar views
    url(
        r'^$',
        views.CalendarCurrentMonthView.as_view(),
        name='current-month'
    ),
    url(
        r'^calendar/(?P<year>\d{4})/(?P<month>\d{2})/$',
        views.CalendarMonthArchiveView.as_view(),
        name='calendar-month'
    ),

    # List views
    url(
        r'^category/(?P<slug>[-\w]+)/$',
        views.EventListCategoryView.as_view(),
        name='category-event-list'
    ),

    # Detail view
    url(
        r'^(?P<year>[0-9]{4})/(?P<month>[0-9]+)/(?P<day>[0-9]+)/(?P<slug>[-\w]+)/$',
        views.EventDetailView.as_view(),
        name='detail'
    ),
]

events_urlpatterns = [
    # List views
    url(
        r'^$',
        views.EventListView.as_view(),
        name='event-list'
    ),
    url(
        r'^archive/$',
        views.EventListArchiveView.as_view(),
        name='event-list-archive'
    ),
    url(
        r'^category/(?P<slug>[-\w]+)/$',
        views.EventListCategoryView.as_view(),
        name='category-event-list'
    ),
    url(
        r'^category/(?P<slug>[-\w]+)/archive/$',
        views.EventListCategoryArchiveView.as_view(),
        name='category-event-list-archive'
    ),

    # Detail view
    url(
        r'^(?P<year>[0-9]{4})/(?P<month>[0-9]+)/(?P<day>[0-9]+)/(?P<slug>[-\w]+)/$',
        views.EventDetailView.as_view(),
        name='detail'
    ),
]

# Default to calendar urlpatterns
urlpatterns = calendar_urlpatterns



from django import template
from django.utils import timezone

from ..models import Event

register = template.Library()


@register.assignment_tag
def get_upcoming_events(count=5, category=None):
    event_list = Event.objects.published().filter(start__gt=timezone.now())

    # Optional filter by category
    if category is not None:
        event_list = event_list.filter(category__slug=category)

    return event_list[:count]



# -*- coding: utf-8 -*-

from django.contrib import admin

from glitter import block_admin
from glitter.admin import GlitterAdminMixin

from .models import Category, Event, UpcomingEventsBlock


@admin.register(Category)
class CategoryAdmin(admin.ModelAdmin):
    search_fields = ('title',)
    prepopulated_fields = {
        'slug': ('title',)
    }


@admin.register(Event)
class EventAdmin(GlitterAdminMixin, admin.ModelAdmin):
    fieldsets = (
        ('Event', {
            'fields': (
                'title', 'category', 'location', 'image', 'summary', 'start', 'end'
            )
        }),
        ('Advanced options', {
            'fields': ('slug',)
        }),
    )
    date_hierarchy = 'start'
    list_display = ('title', 'start', 'end', 'category',)
    list_filter = ('published', 'start', 'category',)
    prepopulated_fields = {
        'slug': ('title',)
    }


block_admin.site.register(UpcomingEventsBlock)
block_admin.site.register_block(UpcomingEventsBlock, 'App Blocks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import glitter.assets.fields


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
        ('glitter_assets', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('title', models.CharField(db_index=True, max_length=100)),
                ('slug', models.SlugField(max_length=100, unique=True)),
            ],
            options={
                'verbose_name_plural': 'Categories',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('published', models.BooleanField(default=True, db_index=True)),
                ('title', models.CharField(db_index=True, max_length=100)),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('location', models.CharField(blank=True, max_length=128)),
                ('summary', models.TextField(help_text='A short sentence description of the event.')),
                ('start', models.DateTimeField(help_text='Start time/date.')),
                ('end', models.DateTimeField(help_text='End time/date.')),
                ('date_url', models.DateField(editable=False, db_index=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('category', models.ForeignKey(to='glitter_events.Category')),
                ('current_version', models.ForeignKey(blank=True, to='glitter.Version', editable=False, null=True)),
                ('image', glitter.assets.fields.AssetForeignKey(blank=True, to='glitter_assets.Image', null=True)),
            ],
            options={
                'ordering': ('start',),
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'edit', 'publish'),
            },
        ),
        migrations.CreateModel(
            name='UpcomingEventsBlock',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('category', models.ForeignKey(blank=True, to='glitter_events.Category', null=True)),
                ('content_block', models.ForeignKey(to='glitter.ContentBlock', editable=False, null=True)),
            ],
            options={
                'verbose_name': 'upcoming events',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.core.urlresolvers import reverse

from glitter.assets.fields import AssetForeignKey
from glitter.mixins import GlitterMixin
from glitter.models import BaseBlock


@python_2_unicode_compatible
class Category(models.Model):
    title = models.CharField(max_length=100, db_index=True)
    slug = models.SlugField(max_length=100, unique=True)

    class Meta:
        ordering = ('title',)
        verbose_name_plural = 'Categories'

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('glitter-events:category-event-list', kwargs={
            'slug': self.slug,
        })


@python_2_unicode_compatible
class Event(GlitterMixin):
    category = models.ForeignKey('glitter_events.Category')
    title = models.CharField(max_length=100, db_index=True)
    slug = models.SlugField(max_length=100, unique_for_date='start')
    location = models.CharField(max_length=128, blank=True)
    image = AssetForeignKey('glitter_assets.Image', null=True, blank=True)
    summary = models.TextField(help_text='A short sentence description of the event.')
    start = models.DateTimeField(help_text='Start time/date.')
    end = models.DateTimeField(help_text='End time/date.')
    date_url = models.DateField(db_index=True, editable=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta(GlitterMixin.Meta):
        ordering = ('start',)

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.date_url = self.start.date()
        super(Event, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('glitter-events:detail', kwargs={
            'year': self.date_url.year,
            'month': str(self.date_url.month).zfill(2),
            'day': str(self.date_url.day).zfill(2),
            'slug': self.slug,
        })


class UpcomingEventsBlock(BaseBlock):
    category = models.ForeignKey('glitter_events.Category', null=True, blank=True)

    class Meta:
        verbose_name = 'upcoming events'



# -*- coding: utf-8 -*-

import datetime

from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.views.generic import ListView
from django.views.generic.dates import DateDetailView, MonthArchiveView

from glitter.mixins import GlitterDetailMixin

from .mixins import CalendarMixin, CategoryMixin, EventsMixin, EventsQuerysetMixin
from .models import Category, Event


class EventDetailView(GlitterDetailMixin, EventsMixin, DateDetailView):
    model = Event
    month_format = '%m'
    date_field = 'date_url'
    allow_future = True

    def get_context_data(self, **kwargs):
        context = super(EventDetailView, self).get_context_data(**kwargs)
        context['current_category'] = self.object.category
        return context


class CalendarCurrentMonthView(CalendarMixin, EventsMixin, MonthArchiveView):
    def get_year(self):
        return str(timezone.now().year)

    def get_month(self):
        return str(timezone.now().month)


class CalendarMonthArchiveView(CalendarMixin, EventsMixin, MonthArchiveView):
    pass


class BaseEventListView(EventsQuerysetMixin, EventsMixin, ListView):
    paginate_by = 10


class EventListView(BaseEventListView):
    def get_queryset(self):
        qs = super(EventListView, self).get_queryset()
        today = datetime.datetime.combine(date=datetime.date.today(), time=datetime.time.min)
        today = timezone.make_aware(today)
        return qs.filter(start__gte=today)


class EventListArchiveView(BaseEventListView):
    template_name_suffix = '_list_archive'

    def get_queryset(self):
        qs = super(EventListArchiveView, self).get_queryset()
        today = datetime.datetime.combine(date=datetime.date.today(), time=datetime.time.min)
        today = timezone.make_aware(today)
        return qs.filter(start__lt=today).order_by('-start')


class EventListCategoryView(CategoryMixin, EventListView):
    template_name_suffix = '_list_category'

    def get_queryset(self):
        qs = super(EventListCategoryView, self).get_queryset()
        self.category = get_object_or_404(Category, slug=self.kwargs['slug'])
        return qs.filter(category=self.category)


class EventListCategoryArchiveView(CategoryMixin, EventListArchiveView):
    template_name_suffix = '_list_category_archive'

    def get_queryset(self):
        qs = super(EventListCategoryArchiveView, self).get_queryset()
        self.category = get_object_or_404(Category, slug=self.kwargs['slug'])
        return qs.filter(category=self.category)



# -*- coding: utf-8 -*-

import calendar
import datetime
from collections import OrderedDict

from django.utils import timezone

from .models import Category, Event


class EventsMixin(object):
    def get_context_data(self, **kwargs):
        context = super(EventsMixin, self).get_context_data(**kwargs)
        context['events_categories'] = True
        context['categories'] = Category.objects.all()
        return context


class CategoryMixin(object):
    def get_context_data(self, **kwargs):
        context = super(CategoryMixin, self).get_context_data(**kwargs)
        context['current_category'] = self.category
        return context


class EventsQuerysetMixin(object):
    model = Event
    queryset = Event.objects.published()


class CalendarMixin(EventsQuerysetMixin):
    allow_future = True
    allow_empty = True
    month_format = '%m'
    date_field = 'date_url'

    def get_time_now(self):
        return timezone.now()

    def get_current_month(self):
        return datetime.date(year=int(self.get_year()), month=int(self.get_month()), day=1)

    def get_context_data(self, **kwargs):
        context = super(CalendarMixin, self).get_context_data(**kwargs)

        context['calendar_headings'] = self.get_calendar_day_names()
        context['calendar_events'] = self.get_events_list()

        return context

    def get_events_list(self):
        current_month = self.get_current_month()
        month_days = OrderedDict()

        cal = calendar.Calendar(firstweekday=calendar.SUNDAY)
        for week in cal.monthdatescalendar(current_month.year, current_month.month):
            for i in week:
                month_days[i] = []

        for obj in self.object_list:
            event_date = obj.start.date()
            month_days[event_date].append(obj)

        return month_days.items()

    def get_calendar_day_names(self):
        calendar_days = []
        day_names = list(calendar.day_name)
        cal = calendar.Calendar(firstweekday=calendar.SUNDAY)

        for i in cal.iterweekdays():
            calendar_days.append(day_names[i])

        return calendar_days
